//
//  BDBadgeCell.m
//  rssbucket
//
// Copyright 2008 Brian Dunagan (brian@bdunagan.com)
//
// MIT License
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without
// restriction, including without limitation the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following
// conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
//

#import "BDBadgeCell.h"

@implementation BDBadgeCell

// Initialize badge variables, based on Apple Mail.
static int BADGE_BUFFER_LEFT = 4;
static int BADGE_BUFFER_SIDE = 3;
static int BADGE_BUFFER_TOP = 16;
static int BADGE_BUFFER_LEFT_SMALL = 3;
static int BADGE_CIRCLE_BUFFER_RIGHT = 5;
static int BADGE_TEXT_HEIGHT = 14;
static int BADGE_X_RADIUS = 7;
static int BADGE_Y_RADIUS = 8;
static int BADGE_TEXT_MINI = 8;
static int BADGE_TEXT_SMALL = 20;

- (id) initWithCoder:(NSCoder *)coder
{
	self = [super init];
	if (self != nil)
	{
		[self setEditable:NO];
		[self setLeaf:YES];
	}
	return self;
}

- (void)encodeWithCoder:(NSCoder *)coder
{
	[super encodeWithCoder:coder];
}

- (void)setBadgeCount:(int)newBadgeCount
{
	badgeCount = newBadgeCount;
}

- (NSColor *)highlightColorInView:(NSView *)controlView
{
	return [NSColor clearColor];
}

- (void)drawInteriorWithFrame:(NSRect)aRect inView:(NSView *)controlView
{
	// Set up badge string and size.
	NSString *badge = [NSString stringWithFormat:@"%d", badgeCount];
	NSSize badgeNumSize = [badge sizeWithAttributes:nil];
	
	// Calculate the badge's coordinates.
	int badgeWidth = badgeNumSize.width + BADGE_BUFFER_SIDE * 2;
	if (badgeNumSize.width < BADGE_TEXT_MINI)
	{
		// The text is too short. Decrease the badge's size.
		badgeWidth = BADGE_TEXT_SMALL;
	}
	int badgeX = aRect.origin.x + aRect.size.width - BADGE_CIRCLE_BUFFER_RIGHT - badgeWidth;
	int badgeNumX = BADGE_BUFFER_LEFT;
	if (badgeNumSize.width < BADGE_TEXT_MINI)
	{
		badgeNumX += BADGE_BUFFER_LEFT_SMALL;;
	}

	// Draw the badge number.
	NSColor *textColor;
	NSColor *badgeColor;
	NSCompositingOperation operation;
	if ([[NSApp mainWindow] isVisible] && [self isHighlighted])
	{
		// The row is selected and the window is in focus.
		textColor = [NSColor alternateSelectedControlColor];
		badgeColor = [NSColor whiteColor];
		operation = NSCompositeDestinationOut;
	}
	else if ([[NSApp mainWindow] isVisible] && ![self isHighlighted])
	{
		// The row is not selected and the window is in focus.
		textColor = [NSColor whiteColor];
		badgeColor = [NSColor colorWithCalibratedRed:.53 green:.60 blue:.74 alpha:1.0];
		operation = NSCompositeSourceOver;
	}
	else if (![[NSApp mainWindow] isVisible] && ![self isHighlighted])
	{
		// The row is not selected and the window is in focus.
		textColor = [NSColor whiteColor];
		badgeColor = [NSColor disabledControlTextColor];
		operation = NSCompositeSourceOver;
	}
	else
	{
		// The row is not selected and the window is in focus.
		textColor = [NSColor disabledControlTextColor];
		badgeColor = [NSColor whiteColor];
		operation = NSCompositeDestinationOut;
	}

	// Draw the text in an image.
	NSImage *textImage = [[NSImage alloc] initWithSize:NSMakeSize(badgeWidth, BADGE_TEXT_HEIGHT)];
	[textImage lockFocus];
	NSDictionary *dict = [[NSMutableDictionary alloc] init];
	[dict setValue:[NSFont fontWithName:@"Helvetica-Bold" size:11] forKey:NSFontAttributeName];
	[dict setValue:textColor forKey:NSForegroundColorAttributeName];
	[badge drawAtPoint:NSMakePoint(badgeNumX, 0) withAttributes:dict];
	[dict release];
	[textImage unlockFocus];

	// Draw the text image in the badge image and onto the source list.
	NSImage *badgeImage = [[NSImage alloc] initWithSize:NSMakeSize(badgeWidth, BADGE_TEXT_HEIGHT)];
	[badgeImage lockFocus];
	[badgeColor set];
	NSRect pathRect = NSMakeRect(0, 0, badgeWidth, BADGE_TEXT_HEIGHT);
	NSBezierPath *path = [NSBezierPath bezierPathWithRoundedRect:pathRect xRadius:BADGE_X_RADIUS yRadius:BADGE_Y_RADIUS];
	[path fill];
	[textImage compositeToPoint:NSZeroPoint operation:operation]; 
	[badgeImage unlockFocus];
	[badgeImage compositeToPoint:NSMakePoint(badgeX, aRect.origin.y + BADGE_BUFFER_TOP) operation:NSCompositeSourceOver];
	
	// Draw NSBrowserCell.
	NSRect labelRect = aRect;
	labelRect.size.width -= badgeWidth;
	[super drawInteriorWithFrame:labelRect inView:controlView];
	
	[textImage release];
	[badgeImage release];
}

@end
